$appDisplayName = "Reflex Reporting"
$appSolutionFileName = "Thor.SPF.ReflexReporting.wsp"

Function Install-ThorApps
{
<# 
.DESCRIPTION
Adds the  Solution to the SharePoint Farm

.EXAMPLE
Install-ThorApps

.Notes
Name: Install-ThorApps
Author: Adrian Bear
Last Edit: August 21st, 2013

# Requires PowerShell Version 2.0 or Version 3.0
# You should run PowerShell Administrator
#>
[CmdletBinding()]
Param()

    Begin {
        Write-Verbose "Entering Begin Block"

        try {

            $SPSnapin = "Microsoft.SharePoint.PowerShell"
            if (Get-PSSnapin $SPSnapin -ErrorAction SilentlyContinue) {
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin already registered and loaded"
            }
            elseif (Get-PSSnapin $SPSnapin -Registered -ErrorAction SilentlyContinue) {
                Add-PSSnapin $SPSnapin
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin is registered and has been loaded for script operation"
            }
            else {
                Write-Error "Microsoft.SharePoint.PowerShell Snappin not found. Exiting function."
                Exit		        
            }

        }
        catch {

            Write-Error "There was an issue loading the Microsoft.SharePoint.PowerShell Snappin. Exiting function."
	        Write-Error $_
            Exit  
                    
        }

        Write-Verbose "Leaving Begin Block"
    }

    
    Process    {
        Write-Verbose "Entering Process Block"
        try
        {
            $farm=[Microsoft.SharePoint.Administration.SPFarm]::Local
            $scriptInvocation = (Get-Variable MyInvocation -Scope 1).Value
            $scriptPath=Split-Path $scriptInvocation.MyCommand.Path
            $menuGroupInstalled=$FALSE
            $menuGroupDeployed=$FALSE
            $appInstalled=$FALSE
            $appDeployed=$FALSE
            $message=""
        
            foreach ($solution in $farm.Solutions)
            {
                if ($solution.DisplayName -eq "thor.spf.licensemanager.wsp")
                {
                    Write-Verbose "Thor License Manager has already been added"
                    $menuGroupInstalled=$TRUE
                    if ($solution.Deployed)
                    {
                        $menuGroupDeployed=$TRUE
                    }
                }
                if ($solution.DisplayName -eq $appSolutionFileName.ToLower())
                {
                    $message = "'" + $appDisplayName + "'has already been added"
                    Write-Verbose $message
                    $appInstalled=$TRUE
                    if ($solution.Deployed)
                    {
                        $appDeployed=$TRUE
                    }
                }
            }

            if ($appInstalled)
            {
                $message = "'" + $appDisplayName + "' is already installed. Please uninstall the current version before re-installing."
                Write-Host $message
            }
            else
                {
                if (-Not($menuGroupInstalled))
                {
                    Write-Verbose "Adding Thor License Manager"
                    $thorLicenseWSPPath=$scriptPath + "\Thor.SPF.LicenseManager.wsp"
                    Add-SPSolution -LiteralPath $thorLicenseWSPPath
                }
        
                if (-Not($appInstalled))
                {
                    $message = "Adding '" + $appDisplayName + "'"
                    Write-Verbose $message
                    $thorAppWSPPath=$scriptPath + "\" + $appSolutionFileName
                    Add-SPSolution -LiteralPath $thorAppWSPPath
                }

                if (-NOT($menuGroupDeployed))
                {
                    Write-Verbose "Deploying Thor License Manager"
                    Install-SPSolution -Identity "thor.spf.licensemanager.wsp" -GACDeployment -Force
                }

                if (-NOT($appDeployed))
                {
                    $message = "Deploying '" + $appDisplayName + "'"
                    Write-Verbose $message
                    Install-SPSolution -Identity $appSolutionFileName.ToLower() -GACDeployment -Force
                }
            }

            WaitForJobToFinish
        }
        catch {

            Write-Error "There was an issue Adding or Installing the solution. Exiting function."
	        Write-Error $_
            Exit  
                    
        }
        Write-Verbose "Leaving Process Block"
    }
}

function WaitForJobToFinish([string]$SolutionFileName)
{ 
    try
    {
        $JobNames = "*solution-deployment*$SolutionFileName*"
        $job = Get-SPTimerJob | ?{ $_.Name -like $JobNames }
        if ($job -eq $null) 
        {
            #Write-Host 'Timer job not found'
        }
        else
        {
            foreach ($jobName in $job)
            {
                $JobFullName = $jobName.Name
                Write-Host ""
                Write-Host -NoNewLine "Waiting to finish job $JobFullName"
    
                while ((Get-SPTimerJob $JobFullName) -ne $null) 
                {
                    Write-Host -NoNewLine .
                    Start-Sleep -Seconds 2
                }
            }
            Write-Host ""
            Write-Host "Finished waiting for job.."
        }
    }
    catch {

        Write-Error "There was an issue waiting for a solution deployment job. Exiting function."
	    Write-Error $_
        Exit  
    }
}

clear
If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))

{   
$arguments = "& '" + $myinvocation.mycommand.definition + "'"
Start-Process powershell -Verb runAs -ArgumentList $arguments
Break
}

Write-Host ""
Write-Host ""
$message = "You are about to install '" + $appDisplayName + "' into your SharePoint farm. Have you read and do you accept the End User License Agreement terms defined in the document 'ThorApps End User Agreement.pdf'"
Write-Host $message
$userInput=Read-Host("Press 'Y' to continue...")
if ($userInput.ToLower() -eq "y")
{
    Install-ThorApps -Verbose:$false
}
else
{
    Write-Host "Operation cancelled."
}
Write-Host ""
Write-Host ""
Write-Host "Script has completed."
#$userInput=Read-Host("Press the Enter key to continue...")
