$appDisplayName = "Reflex Reporting"
$appSolutionFileName = "Thor.SPF.ReflexReporting.wsp"

Function Remove-ThorApps
{
<# 
.DESCRIPTION
Removes the  Project Solution from the SharePoint Farm

.EXAMPLE
Remove-ThorApps

.NotesPIE
Author: Adrian Bear
Last Edit: August 21st, 2013

# Requires PowerShell Version 2.0 or Version 3.0
# You should run PowerShell Administrator
#>
[CmdletBinding()]
Param()

    Begin {
        Write-Verbose "Entering Begin Block"

        try {

            $SPSnapin = "Microsoft.SharePoint.PowerShell"
            if (Get-PSSnapin $SPSnapin -ErrorAction SilentlyContinue) {
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin already registered and loaded"
            }
            elseif (Get-PSSnapin $SPSnapin -Registered -ErrorAction SilentlyContinue) {
                Add-PSSnapin $SPSnapin
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin is registered and has been loaded for script operation"
            }
            else {
                Write-Error "Microsoft.SharePoint.PowerShell Snappin not found. Exiting function."
                Exit		        
            }

        }
        catch {

            Write-Error "There was an issue loading the Microsoft.SharePoint.PowerShell Snappin. Exiting function."
	        Write-Error $_
            Exit  
                    
        }

        Write-Verbose "Leaving Begin Block"
    }

    
    Process    {
        Write-Verbose "Entering Process Block"

        try
        {
            $farm=[Microsoft.SharePoint.Administration.SPFarm]::Local
            $scriptInvocation = (Get-Variable MyInvocation -Scope 1).Value
            $scriptPath=Split-Path $scriptInvocation.MyCommand.Path
            $menuGroupInstalled=$FALSE
            $menuGroupDeployed=$FALSE
            $appInstalled=$FALSE
            $appDeployed=$FALSE
            $otherThorAppsExist=$FALSE
        
            foreach ($solution in $farm.Solutions)
            {
                if ($solution.DisplayName -eq "thor.spf.licensemanager.wsp")
                {
                    $menuGroupInstalled=$TRUE
                    if ($solution.Deployed)
                    {
                        $menuGroupDeployed=$TRUE
                    }
                }
                elseif ($solution.DisplayName -eq $appSolutionFileName.ToLower())
                {
                    $appInstalled=$TRUE
                    if ($solution.Deployed)
                    {
                        $appDeployed=$TRUE
                    }
                }
                elseif ($solution.DisplayName.StartsWith("thor.", "CurrentCultureIgnoreCase"))
                {
                    $otherThorAppsExist=$TRUE
                }

            }


            if ($menuGroupDeployed -and -NOT($otherThorAppsExist))
            {
                Write-Verbose "Retracting Thor License Manager"
                Uninstall-SPSolution -Identity "thor.spf.licensemanager.wsp" -Confirm:$false
            }

            $message = ""
            if ($appDeployed)
            {
                $message = "Retracting '" + $appDisplayName + "'"
                Write-Verbose $message
                Uninstall-SPSolution -Identity $appSolutionFileName -Confirm:$false
            }
        
            WaitForJobToFinish

            if ($appInstalled)
            {
                $message = "Removing '" + $appDisplayName + "'"
                Write-Verbose $message
                Remove-SPSolution -Identity $appSolutionFileName.ToLower() -Confirm:$false -Force
            }

            if ($menuGroupInstalled)
            {
                if (-NOT($otherThorAppsExist))
                {
                    Write-Verbose "Removing Thor License Manager"
                    Remove-SPSolution -Identity "thor.spf.licensemanager.wsp" -Confirm:$false -Force
                }
                else
                {
                    Write-Host "Skipping the removal of 'Thor License Manager' because other Thor Apps exist that may depend on it."
                }
            }
        }
        catch {

            Write-Error "There was an issue Removing or Retracting the solution. Exiting function."
	        Write-Error $_
            Exit  
                    
        }
        Write-Verbose "Leaving Process Block"
    }
}

function WaitForJobToFinish([string]$SolutionFileName)
{ 
    try
    {
        $JobNames = "*solution-deployment*$SolutionFileName*"
        $job = Get-SPTimerJob | ?{ $_.Name -like $JobNames }
        if ($job -eq $null) 
        {
            #Write-Host 'Timer job not found'
        }
        else
        {
            foreach ($jobName in $job)
            {
                $JobFullName = $jobName.Name
                Write-Host ""
                Write-Host -NoNewLine "Waiting to finish job $JobFullName"
    
                while ((Get-SPTimerJob $JobFullName) -ne $null) 
                {
                    Write-Host -NoNewLine .
                    Start-Sleep -Seconds 2
                }
            }
            Write-Host ""
            Write-Host "Finished waiting for job.."
        }
    }
    catch {

        Write-Error "There was an issue waiting for a solution deployment job. Exiting function."
	    Write-Error $_
        Exit  
    }
}


clear
If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))

{   
$arguments = "& '" + $myinvocation.mycommand.definition + "'"
Start-Process powershell -Verb runAs -ArgumentList $arguments
Break
}

Write-Host ""
Write-Host ""
$message = "You are about to remove '" + $appDisplayName + "' from your SharePoint farm."
Write-Host $message
$userInput=Read-Host("Press 'Y' to continue...")
if ($userInput.ToLower() -eq "y")
{
    Remove-ThorApps -Verbose:$false
}
else
{
    Write-Host "Operation cancelled."
}
Write-Host ""
Write-Host ""
Write-Host "Script has completed."
#$userInput=Read-Host("Press the Enter key to continue...")
